package com.sap.cloud.sample.authentication.integrationtest;

import static org.hamcrest.Matchers.containsString;

import java.io.IOException;

import org.apache.http.auth.AuthScope;
import org.htmlunit.FailingHttpStatusCodeException;
import org.htmlunit.WebClient;
import org.htmlunit.html.HtmlPage;
import org.htmlunit.httpclient.HtmlUnitUsernamePasswordCredentials;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * Integration test using HtmlUnit.
 */
public class AuthenticationIntegrationTest {
    private static String serverUrl, username, password;

    /**
     * Take a provided server URL usually fed in from outside through the build process or default to the local server
     * as provided through Eclipse to run the integration test against.
     */
    @BeforeClass
    public static void setupSuite() {
        // Get integration test server URL
        serverUrl = System.getProperty("integration.test.server.url");
        if (serverUrl == null) {
            serverUrl = "http://localhost:8080";
        }
        System.out.println("Running against " + serverUrl);

        // Guess integration test scenario, set user name and password
        if (serverUrl.contains("localhost")) {
            username = "john";
            password = "doe";
        } else {
            username = System.getProperty("sap.cloud.username");
            password = System.getProperty("sap.cloud.password");
        }
        System.out.println("Using user " + username);
    }

    /**
     * Call the main servlet and check that it contains the expected content.
     */
    @Test
    public void testAuthenticationServlet() throws IOException {
        // Call servlet
        try (final WebClient webClient = new WebClient()) {
            final HtmlPage page = webClient.getPage(serverUrl + "/authentication/");
            final String pageAsText = page.asNormalizedText();
            Assert.assertThat(pageAsText, containsString("Authentication"));
        }
    }

    /**
     * Try to login, but without credentials which should fail.
     */
    @Test(expected = FailingHttpStatusCodeException.class)
    public void testBasicAuthenticationRequired() throws IOException {
        // Call protected servlet without authentication
        try (final WebClient webClient = new WebClient()) {
            webClient.getPage(serverUrl + "/authentication/protected");
        }
    }

    /**
     * Try to login, but with wrong password which should fail.
     */
    @Test(expected = FailingHttpStatusCodeException.class)
    public void testWrongPassword() throws IOException {
        // Call protected servlet with wrong authentication
        try (final WebClient webClient = new WebClient()) {
            webClient.getCredentialsProvider().setCredentials(AuthScope.ANY,
                    new HtmlUnitUsernamePasswordCredentials(username, new char[] {' '}));
            webClient.getPage(serverUrl + "/authentication/protected");
        }
    }

    /**
     * Try to login with correct credentials which should pass.
     */
    @Test
    public void testSuccessfulLogin() throws IOException {
        // Call protected servlet with correct authentication
        try (final WebClient webClient = new WebClient()) {
            webClient.getCredentialsProvider().setCredentials(AuthScope.ANY,
                    new HtmlUnitUsernamePasswordCredentials(username, password.toCharArray()));
            final HtmlPage page = webClient.getPage(serverUrl + "/authentication/protected");
            final String pageAsText = page.asNormalizedText();
            Assert.assertThat(pageAsText.toLowerCase(), containsString("welcome"));
        }
    }

    /**
     * Try to logout which should pass.
     */
    @Test
    public void testSuccessfulLogout() throws IOException {
        // Call logout servlet with correct authentication
        try (final WebClient webClient = new WebClient()) {
            final HtmlPage page = webClient.getPage(serverUrl + "/authentication/logout");
            final String pageAsText = page.asNormalizedText();
            Assert.assertThat(pageAsText, containsString("No user logged in anymore"));
        }

    }
}
